#!/bin/bash
#
# bash-common-api:lock-api.sh
#

## Initial version:
## Copyright (C) 2009 Przemyslaw Pawelczyk <przemoc@gmail.com>
##
## This script is licensed under the terms of the MIT license.
## https://opensource.org/licenses/MIT

#
# Lockable script boilerplate
#
# https://stackoverflow.com/questions/1715137/what-is-the-best-way-to-ensure-only-one-instance-of-a-bash-script-is-running
#

### HEADER ###

if [ -z "${LOCKFILE:-}" ] ; then
  echo '***Fatal: LOCKFILE is not set.' >&2
  exit 1
fi
LOCKFD=99

# PRIVATE
_lock()             { flock -"$1" "$LOCKFD"; }
_no_more_locking()  { _lock u; _lock xn && rm -f "$LOCKFILE"; }
_prepare_locking()  { eval "exec $LOCKFD>\"$LOCKFILE\""; trap _no_more_locking EXIT; }

# ON START
_prepare_locking

# PUBLIC
exlock_now()        { _lock xn; }  # obtain an exclusive lock immediately or fail
exlock()            { _lock x; }   # obtain an exclusive lock
shlock()            { _lock s; }   # obtain a shared lock
unlock()            { _lock u; }   # drop a lock

###

function _lock_api_check_screen_ {
  local parent_process=
  parent_process="$( ps -fp$PPID | head -n 2 | tail -n 1 )"

  if grep -E --quiet SCREEN <<< "${parent_process}" ; then
    # Screen is parent process
    return 0
  else
    # Screen is not parent process
    return 1
  fi
}

function lock_api_check_screen {
  _lock_api_check_screen_ || return $?
}

function screen_error_display {
  if _lock_api_check_screen_ ; then
    sleep 15 # Allow to read error (when screen is running)
  fi
}

### BEGIN OF SCRIPT ###

# Simplest example is avoiding running multiple instances of script.
if ! exlock_now ; then
  cat <<EOF >&2
*** Lock already taked: ${LOCKFILE}
EOF
  screen_error_display
  exit 1
fi

# Remember! Lock file is removed when one of the scripts exits and it is
#           the only script holding the lock or lock is not acquired at all.

