#!/bin/bash
#
# freebox-tools:bin/api/freebox-bash-api.sh
#
if [ "$( type -t echofatal )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function echofatal not found.' >&2
  exit 100
fi

declare -gr FREEBOX_URL_NO_PROTO='mafreebox.freebox.fr'

declare __API_VERSION__=
declare __API_BASE_URL__=
declare __SESSION_TOKEN__=

declare __FREEBOX_PUBLIC_USE_HTTPS__='false'
declare __FREEBOX_PUBLIC_DOMAIN_URL__=
declare __FREEBOX_PUBLIC_HTTPS_PORT__=

declare __NOT_TRUSTED_ALLOWED__='false'
declare __HTTPS_ONLY__='true'

_PRINT_ERROR_ONCE_HTTPS_NOT_WORKING_PROPERLY="$( mktemp )" || exit $?
declare -gr _PRINT_ERROR_ONCE_HTTPS_NOT_WORKING_PROPERLY

function response_check_success {
  local -r response="$1"

  if [ -z "${response}" ] ; then
    echofatal 'Freebox non trouvée'
    return 1
  fi

  local value=
  value="$( jq -r '.success' <<<"${response}" )"
  if [ "${value}" != 'true' ]; then
    echoerror "$( jq -r '.msg' <<<"${response}" )"
    jq '.' <<<"${response}" >&2
    return 199
  fi
  return 0
}

function _print_error_once_https_not_working_properly {
  local _done_=
  read -r _done_ <"${_PRINT_ERROR_ONCE_HTTPS_NOT_WORKING_PROPERLY}"
  if [ -n "${_done_}" ] ; then
    return 0
  fi

  cat <<EOF >&2
* Attention: Freebox non trouvée en HTTPS....

- Elle n'est peut-être pas accessible en ce moment (Redémarrage, pas sur le
  même réseau, ...)
EOF

  if [ "${__NOT_TRUSTED_ALLOWED__}" != 'true' ] ; then
    cat <<EOF >&2
- Elle n'est pas configurée pour le https,
- Elle n'a pas de certificat valide:
  - Ajouter la clé à votre magasin de confiance (choix le plus sécurisé, pas simple à mettre en oeuvre),
  - Ou envisagez d'ajouter de l'option --insecure
EOF

  fi
  echo 'done' >"${_PRINT_ERROR_ONCE_HTTPS_NOT_WORKING_PROPERLY}"
}

function _print_error_once_enable_http {
  cat <<EOF >&2
  - Ou autoriser l'usage du http (non sécurisé) à l'aide de l'option: --allow-http (le moins sécurisé)
EOF
}

function freebox_local_only_api_version_json { # API
  __FREEBOX_PUBLIC_USE_HTTPS__='false'
  freebox_api_version_json || return $?
}

function freebox_api_version_json { # API
  _do_curl 'GET' '/api_version' '' || return $?
}

function freebox_api_init_api_access { # API
  __FREEBOX_PUBLIC_USE_HTTPS__="${1:-false}"
  __FREEBOX_PUBLIC_DOMAIN_URL__="${2:-UNSET}"
  __FREEBOX_PUBLIC_HTTPS_PORT__="${3:-UNSET}"

  local answer=
  answer="$( freebox_api_version_json )" || return $?

  __API_VERSION__="$( jq -r '.api_version' <<<"${answer}" | sed 's/\..*//')" || return $?
  __API_BASE_URL__="$( jq -r '.api_base_url' <<<"${answer}" )" || return $?
}

function _do_curl {
  local -r method_request="$1"
  local -r api_uri="$2"
  local -r data="${3:-}" # Optionnal

  local -a curl_options=('-s')
  curl_options+=('--http1.1')

  if [ "${__NOT_TRUSTED_ALLOWED__}" = 'true' ] || [ "${FREEBOX_INSECURE:-}" = 'true' ] ; then
    curl_options+=('-k')
  fi

  case "${method_request}" in
    'GET' ) ;;
    'POST') curl_options+=(-X 'POST' ) ;;
    'PUT') curl_options+=(-X 'PUT' ) ;;
    *)
      echo "ERROR: Unsupported request method '${method_request}" >&2
      return 100
      ;;
  esac

  if [ -n "${__SESSION_TOKEN__}" ] ; then
    curl_options+=(-H "X-Fbx-App-Auth: ${__SESSION_TOKEN__}")
  fi
  if [ -n "${data}" ] ; then
    if jq -c '.' <<<"${data}" 1>/dev/null; then
      # JSON format found
      curl_options+=(-H "Content-Type: application/json")
      curl_options+=(--data-raw "${data}")
    else
      curl_options+=(--data "${data}")
    fi
  fi

  local url=
  local answer=

  # External URL https
  if [ "${__FREEBOX_PUBLIC_USE_HTTPS__}" = 'true' ] ; then
    url="https://${__FREEBOX_PUBLIC_DOMAIN_URL__}:${__FREEBOX_PUBLIC_HTTPS_PORT__}${api_uri}"
    answer="$(
      set +e
      if [ "${FREEBOX_DEBUG_CURL:-}" = 'true' ] ; then echowarn 'External URL https' ; set -x ; fi
      curl "${curl_options[@]}" "${url}"
      )" || true
  fi

  # Internal URL https
  if [ -z "${answer}" ] ; then
    url="https://${FREEBOX_URL_NO_PROTO}${api_uri}"
    answer="$(
      set +e
      if [ "${FREEBOX_DEBUG_CURL:-}" = 'true' ] ; then echowarn 'Internal URL https' ; set -x ; fi
      curl "${curl_options[@]}" "${url}"
      )" || true
  fi

  # Internal URL http
  if [ -z "${answer}" ] ; then
    _print_error_once_https_not_working_properly || return $?

    if [ "${__HTTPS_ONLY__}" != 'true' ] || [ "${FREEBOX_HTTPS_ONLY:-}" = 'true' ]; then
      url="http://${FREEBOX_URL_NO_PROTO}${api_uri}"
      answer="$(
        set +e
        if [ "${FREEBOX_DEBUG_CURL:-}" = 'true' ] ; then echowarn 'Internal URL http' ; set -x ; fi
        curl "${curl_options[@]}" "${url}"
        )" || true
    else
      _print_error_once_enable_http || return $?
    fi
  fi

  if [ -n "${answer}" ] ; then
    echo "${answer}"
  else
    echofatal "No answer for '${url}'"
    return 100
  fi
}

function normalize_uri {
  local -r api_uri="$1" # R/W

  if [ "${api_uri:0:1}" == '/' ] ; then
    echo "${api_uri:1}"
  else
    echo "${api_uri}"
  fi
}

function __freebox_api_request__ {
  local -r method_request="$1"
  local -r api_url="$2"
  local -r data="$3"

  if [ -z "${__API_BASE_URL__}" ] ; then
    echofatal "Votre Freebox n'a pas été trouvée. (__API_BASE_URL__ not define)"
    return 110
  fi
  if [ -z "${__API_VERSION__}" ] ; then
    echofatal "${FUNCNAME[0]}() - ERROR. __API_VERSION__ not define."
    return 110
  fi

  local _normalized_api_uri_=
  _normalized_api_uri_="$( normalize_uri "${api_url}" )" || return $?

  local answer=
  answer="$( _do_curl "${method_request}" "${__API_BASE_URL__}v${__API_VERSION__}/${_normalized_api_uri_}" "${data}" )"

  response_check_success "${answer}" || return $?
  echo "${answer}"
}

function freebox_api_get { # API
  local -r api_url="$1"
  local -r data="${2-}"
  local -i i=0

  local result=
  local -i request_rc=0
  # shellcheck disable=SC2034 # $i not used
  for i in {1..3} ; do
    result="$( __freebox_api_request__ 'GET' "${api_url}" "${data}" )"
    request_rc=$?
    if [ "${request_rc}" -eq 0 ] ; then
      echo "${result}"
      return 0
    fi
    sleep 1
  done

  echofatal "${FUNCNAME[0]}() - GET Error: '${api_url}' '${data}' (${request_rc})"
  return "${request_rc}"
}

function freebox_api_get_with_result_if_success { # API
  local -r api_url="$1"
  local -r data="${2-}"

  local json=
  json="$( freebox_api_get "${api_url}" "${data}" )" || return $?

  if [ -n "${json}" ] ; then
    if [ "$( jq '.success' <<<"${json}" )" = 'true' ] ; then
      jq -c '.result' <<<"${json}" || return $?
    else
      return 254
    fi
  else
    return 255
  fi
}

function freebox_api_post { # API
  local -r api_url="$1"
  local -r data="${2:-}"

  __freebox_api_request__ 'POST' "${api_url}" "${data}" || return $?
}

function freebox_api_put { # API
  local -r api_url="$1"
  local -r data="$2"

  __freebox_api_request__ 'PUT' "${api_url}" "${data}" || return $?
}

function freebox_api_put_with_result_if_success { # API
  local -r api_url="$1"
  local -r data="${2-}"

  local json=
  json="$( freebox_api_put "${api_url}" "${data}" )" || return $?

  if [ -n "${json}" ] ; then
    if [ "$( jq '.success' <<<"${json}" )" = 'true' ] ; then
      jq -r '.result' <<<"${json}" || return $?
    else
      return 254
    fi
  else
    return 255
  fi
}

function freebox_api_login { # API
  local -r p_app_id="$1"
  local -r p_app_token="$2"

  local answer=
  answer="$( freebox_api_get 'login' )" || return $?

  local challenge=
  challenge="$( jq -r '.result.challenge' <<<"${answer}" )" || return $?

  local password=
  password="$( echo -n "${challenge}" | openssl dgst -sha1 -hmac "${p_app_token}" | sed  's/^(stdin)= //' )" || return $?

  answer="$( freebox_api_get '/login/session/' "{\"app_id\":\"${p_app_id}\", \"password\":\"${password}\" }" )" || return $?
  __SESSION_TOKEN__="$( jq -r '.result.session_token' <<<"${answer}" )" || return $?
}

function freebox_api_authorize_application {
  local -r p_app_id="$1"
  local -r p_app_name="$2"
  local -r p_app_version="$3"
  local -r p_device_name="$4"

  local data=
  data="$(
    jq -c --null-input \
     --arg id "${p_app_id}" \
     --arg name "${p_app_name}" \
     --arg version "${p_app_version}" \
     --arg device "${p_device_name}" \
     '{ app_id: $id, app_name: $name, app_version: $version, device_name: $device }'
   )" || return $?

  local answer=
  answer="$( freebox_api_get 'login/authorize' "${data}" )"
  [ -z "${answer}" ] && return 120

  local app_token=
  app_token="$( jq -r '.result.app_token' <<<"${answer}" )" || return $?

  local track_id=
  track_id="$( jq -r '.result.track_id' <<<"${answer}" )" || return $?

  cat <<EOF >&2

Donnez les droits à l'application en suivant les instructions sur l'écran LCD de la Freebox...

EOF

  local status='pending'
  while [ "${status}" = 'pending' ] ; do
    sleep 5
    answer="$( freebox_api_get "login/authorize/${track_id}" )"
    status="$( jq -r '.result.status' <<<"${answer}" )"
  done

  echo "Etat de l'autorisation: ${status}" >&2
  [ "${status}" != 'granted' ] && return 199

  echo >&2
  jq -c --null-input \
    --arg id "${p_app_id}" \
    --arg token "${app_token}" \
    '{ my_freebox_app_id: $id, my_freebox_app_token: $token }' || return $?
}

function set_insecure { # API
  __NOT_TRUSTED_ALLOWED__='true'
}

function set_allow_http { # API
  __HTTPS_ONLY__='false'
}
