#!/bin/bash
#
# freebox-tools:bin/api/freebox-config-api.sh
#
# Manage local configuration
#
if [ "$( type -t is_variable_define_and_not_empty_safe )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function is_variable_define_and_not_empty_safe not found.' >&2
  exit 1
fi
if [ "$( type -t load_vars_from_json_file )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function load_vars_from_json_file not found.' >&2
  exit 1
fi

declare -gr FREEBOX_CONFIG_DIRECTORY="${HOME}/.config/freebox"
declare -gr FREEBOX_CONFIG_JSON_FILE="${FREEBOX_CONFIG_DIRECTORY}/freebox.json"

declare -gr FREEBOX_DEPRECATED_CONFIG_APP_ID="${FREEBOX_CONFIG_DIRECTORY}/appid"
declare -gr FREEBOX_DEPRECATED_CONFIG_APP_TOKEN="${FREEBOX_CONFIG_DIRECTORY}/apptoken"
declare -gr FREEBOX_DEPRECATED_CONFIG_PUBLIC_HTTPS="${FREEBOX_CONFIG_DIRECTORY}/https"
declare -gr FREEBOX_DEPRECATED_CONFIG_PUBLIC_DOMAIN_URL="${FREEBOX_CONFIG_DIRECTORY}/domain"
declare -gr FREEBOX_DEPRECATED_CONFIG_PUBLIC_HTTPS_PORT="${FREEBOX_CONFIG_DIRECTORY}/port"

declare -gr _APP_ID_='freebox-tools'
declare -gr _APP_NAME_='FreeBox Toolbox (bash)'
declare -gr _APP_VERSION_='2021.12.21'

#
# Verify configuration
#
function __freebox_config_check__ {
  if ! is_variable_define_and_not_empty_safe 'my_freebox_app_id' ; then
    echoerror "my_freebox_app_id is not define"
    return 100
  fi

  if ! is_variable_define_and_not_empty_safe 'my_freebox_app_token' ; then
    echoerror "my_freebox_app_token is not define"
    return 100
  fi
}

#
# Save configuration
#
function __save_token_from_freebox__ {
  local -r _token_data_="$1" # '{my_freebox_app_id: STRING, my_freebox_app_token: STRING }'
  local -r _freebox_info_="$2" # '{MY_FREEBOX_PUBLIC_DOMAIN_URL: STRING, MY_FREEBOX_PUBLIC_HTTPS: BOOL, MY_FREEBOX_PUBLIC_DOMAIN_URL: NUMBER }'

  mkdir -vp "${FREEBOX_CONFIG_DIRECTORY}" || return $?
  jq --slurp -S '.[0] + .[1]' <<<"${_token_data_}${_freebox_info_}" >"${FREEBOX_CONFIG_JSON_FILE}" || return $?

  __freebox_config_load__ || return $?
  __freebox_config_check__ || return $?
}

function freebox_config_init_token {
  # fill _API_VERSION and _API_BASE_URL variables
  freebox_api_init_api_access || return $?

  # Negociate new token
  local new_token_data=
  new_token_data="$(
    freebox_api_authorize_application "${_APP_ID_}" "${_APP_NAME_}" "${_APP_VERSION_}" "$( hostname )"
    )" || return $?

  if [ -z "${new_token_data}" ]  ; then
    echo 'Problème lors de la récupération du token.'
    return 100
  fi

  local freebox_info_json=
  freebox_info_json="$( freebox_local_only_api_version_json )" || return $?

  echoinfo 'Freebox'
  jq '.' <<<"${freebox_info_json}" >&2

  local freebox_conf_info_json=
  freebox_conf_info_json="$( jq -c '{
  MY_FREEBOX_PUBLIC_DOMAIN_URL: .api_domain,
  MY_FREEBOX_PUBLIC_HTTPS: .https_available ,
  MY_FREEBOX_PUBLIC_HTTPS_PORT: .https_port
}' <<<"${freebox_info_json}" )" || return $?

  echoinfo 'Freebox config'
  jq '.' <<<"${freebox_conf_info_json}" >&2

   __save_token_from_freebox__ "${new_token_data}" "${freebox_conf_info_json}" || return $?
  echookay "${FREEBOX_CONFIG_DIRECTORY} updated." >&2
  cat <<EOF >&2

  Pour accéder à toutes les fonctionnalités de la Freebox vous devez ajouter des droits
  à cette application dans l'interface de la Freebox

EOF
}

#
# Load configuration
#
function __freebox_config_load__ { # _freebox_config_load
  # Mandatory configuration
  declare -g my_freebox_app_id=
  declare -g my_freebox_app_token=

  # Optionnal configuration
  declare -g MY_FREEBOX_PUBLIC_HTTPS=
  declare -g MY_FREEBOX_PUBLIC_DOMAIN_URL=
  declare -g MY_FREEBOX_PUBLIC_HTTPS_PORT=

  if [ -f "${FREEBOX_CONFIG_JSON_FILE}" ] ; then
    local save="${MESSAGE_API_QUIET:-}"
    MESSAGE_API_QUIET='true'
    load_vars_from_json_file "${FREEBOX_CONFIG_JSON_FILE}" '.' || return $?
    MESSAGE_API_QUIET="${save}"
  else
    echowarn "Can't find: '${FREEBOX_CONFIG_JSON_FILE}'"

    # Try to load previous configuration format
    if [ -f "${FREEBOX_DEPRECATED_CONFIG_APP_ID}" ] ; then
      read -r my_freebox_app_id < "${FREEBOX_DEPRECATED_CONFIG_APP_ID}" || return $?
    fi
    if [ -f "${FREEBOX_DEPRECATED_CONFIG_APP_TOKEN}" ] ; then
      read -r my_freebox_app_token < "${FREEBOX_DEPRECATED_CONFIG_APP_TOKEN}" || return $?
    fi
    if [ -f "${FREEBOX_DEPRECATED_CONFIG_PUBLIC_HTTPS}" ] ; then
      read -r MY_FREEBOX_PUBLIC_HTTPS < "${FREEBOX_DEPRECATED_CONFIG_PUBLIC_HTTPS}" || exit 1
    else
      MY_FREEBOX_PUBLIC_HTTPS='false'
    fi
    if [ -f "${FREEBOX_DEPRECATED_CONFIG_PUBLIC_DOMAIN_URL}" ] ; then
      read -r MY_FREEBOX_PUBLIC_DOMAIN_URL < "${FREEBOX_DEPRECATED_CONFIG_PUBLIC_DOMAIN_URL}" || exit 1
    fi
    if [ -f "${FREEBOX_DEPRECATED_CONFIG_PUBLIC_HTTPS_PORT}" ] ; then
      read -r MY_FREEBOX_PUBLIC_HTTPS_PORT < "${FREEBOX_DEPRECATED_CONFIG_PUBLIC_HTTPS_PORT}" || exit 1
    fi
  fi

  if [ -z "${MY_FREEBOX_PUBLIC_HTTPS}" ] ; then
    MY_FREEBOX_PUBLIC_HTTPS='false'
  fi

  declare -gr my_freebox_app_id
  declare -gr my_freebox_app_token
  declare -gr MY_FREEBOX_PUBLIC_HTTPS
  declare -gr MY_FREEBOX_PUBLIC_DOMAIN_URL
  declare -gr MY_FREEBOX_PUBLIC_HTTPS_PORT
}

#
# Freebox load configuration and login
#

function freebox_connect {
  __freebox_config_load__ || return $?
  __freebox_config_check__ || return $?

  # fill _API_VERSION and _API_BASE_URL variables
  freebox_api_init_api_access "${MY_FREEBOX_PUBLIC_HTTPS}" "${MY_FREEBOX_PUBLIC_DOMAIN_URL}" "${MY_FREEBOX_PUBLIC_HTTPS_PORT}" || return $?

  # login
  freebox_api_login "${my_freebox_app_id}" "${my_freebox_app_token}" || return $?
}

