#!/bin/bash
#
# freebox-tools:bin/api/freebox-tools-api.sh
#
if [ "$( type -t P )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function P not found.' >&2
  exit 1
fi

declare -gr __JQ_NORMALIZE__="${PROJECT_HOME}/bin/api/normalize.jq"

declare -g param_active=
declare -g param_family=

#
# Freebox Tools
#
function freebox_reboot {
  freebox_api_get '/system/reboot' '{}' | jq . || return $?
}

function __hosts_filter_active__ {
  case "${param_active}" in
    '')
      jq '.' || return $?
      ;;

    'true'|'false')
      jq  -c --argjson active "${param_active}" \
'{
  success: .success,
  result: [
    .result // [] |
      .[] | select( .l3connectivities // [] | .[] | .active==$active )
  ]
}' || return $?
      ;;

    *)
      echofatal "--active must be empty, true or false. Found: '${param_active}'"
      return 100
      ;;
  esac
}

function __hosts_filter_family__ {
  case "${param_family}" in
    '')
      jq '.' || return $?
      ;;

    'ipv4'|'ipv6')
      jq -c --arg family "${param_family}" \
'{
  success: .success,
  result: [
    .result // [] |
      .[] | del( select( .l3connectivities // [] | .[] | .af != $family ) ) // empty
  ]
}' || return $?
      ;;

    *)
      echofatal "--family must be empty, ipv4 or ipv6. Found: '${param_family}'"
      return 100
      ;;
  esac
}

function jq_normalize {
  jq -S -f "${__JQ_NORMALIZE__}" || return $?
}

function __hosts_filters__ {
  __hosts_filter_family__ | __hosts_filter_active__ | jq_normalize || return $?
}

function get_hosts_json {
  freebox_api_get '/lan/browser/pub' | __hosts_filters__ | jq '. | .result |= ( sort_by( .id ) )' || return $?
}

#
# MAC Addresses
#

function freebox_show_mac_addresses_json {
  get_hosts_json |
    jq '[ .result[].l2ident.id ]' |
    jq_normalize || return $?
}

function freebox_show_mac_addresses_text {
  freebox_show_mac_addresses_json | jq -r '.[]' || return $?
}

#
# display commands: formatted lines
#

declare -r DEFAULT_WHITE_PLACE_HOLDER='                                                                '
declare -r DEFAULT_SEPARATOR='\t'

declare -g WHITE_PLACE_HOLDER="${DEFAULT_WHITE_PLACE_HOLDER}"
declare -g SEPARATOR="${DEFAULT_SEPARATOR}"
declare -g END_SEPARATOR=''

function _freebox_display_hosts_json_raw_ {
  local hosts_json=
  hosts_json="$( get_hosts_json )" || return $?

  jq -c '[ .result[] | {
  names: ( .names // [] | sort_by( .source ) ),
  vendor_name: .vendor_name,
  l2ident: .l2ident,
  l3connectivities: .l3connectivities,
  host_type: .host_type
} ]' <<<"${hosts_json}" || return $?
}

function freebox_display_hosts_json {
  _freebox_display_hosts_json_raw_ | jq_normalize || return $?
}

function _freebox_display_hosts_text_entry_ {
  local -r host_answer="$1"

  local host_ipv4=
  host_ipv4="$( jq -r '[ .l3connectivities // [] | .[] | select( .af == "ipv4") ][0].addr' <<<"${host_answer}" )${WHITE_PLACE_HOLDER}" || return $?

  local host_ipv6=
  host_ipv6="$( jq -r '[ .l3connectivities // [] | .[] | select( .af == "ipv6") ][0].addr' <<<"${host_answer}" )${WHITE_PLACE_HOLDER}" || return $?

  local host_mac=
  host_mac="$( jq -r ".l2ident.id" <<<"${host_answer}" )" || return $?

  local host_name=
  host_name="$( jq -r ".names[0].name" <<<"${host_answer}" )" || return $?
  host_name="${host_name}${WHITE_PLACE_HOLDER}${WHITE_PLACE_HOLDER}"

  local host_type=
  host_type="$( jq -r ".host_type" <<<"${host_answer}" )" || return $?
  host_type="${host_type}${WHITE_PLACE_HOLDER}"

  local host_vendor=
  host_vendor="$( jq -r ".vendor_name" <<<"${host_answer}" )" || return $?
  host_vendor="${host_vendor}${WHITE_PLACE_HOLDER}${WHITE_PLACE_HOLDER}"

  echo -en "${host_ipv4:0:15}${SEPARATOR}"
  echo -en "${host_ipv6:0:26}${SEPARATOR}"
  echo -en "${host_mac,,}${SEPARATOR}"
  echo -en "${host_type:0:20}${SEPARATOR}"
  echo -en "${host_vendor:0:64}${SEPARATOR}"
  echo -en "${host_name:0:64}"
  echo -e  "${END_SEPARATOR}"
}

function freebox_display_hosts_text {
  local json=
  json="$( freebox_display_hosts_json | jq -c '.[]' )" || return $?
  local -r json

  local host_answer=
  while read -r host_answer; do
    if [ -n "${host_answer}" ] ; then
      _freebox_display_hosts_text_entry_ "${host_answer}" || return $?
    fi
  done <<<"${json}"
}
