#!/bin/bash -e
#
# freebox-tools:bin/freebox-internet-access.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

# shellcheck disable=SC1091
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit $?

declare -r FREEBOX_BIN="${PROJECT_HOME}/bin/freebox.sh"
#eclare -g __network_control_profile_id__=

function usage {
  cat <<EOF
Assistant pour votre Freebox permettant de gérer l’accès internet de certain ordinateurs
ou téléphones.

Usage:
  $( basename "${BASH_SOURCE[0]}" ) --help
  $( basename "${BASH_SOURCE[0]}" ) --list
  $( basename "${BASH_SOURCE[0]}" ) --mac-address ADR
  $( basename "${BASH_SOURCE[0]}" ) --mac-address ADR --enable
  $( basename "${BASH_SOURCE[0]}" ) --mac-address ADR --disable
  $( basename "${BASH_SOURCE[0]}" ) --host-name NOM
  $( basename "${BASH_SOURCE[0]}" ) --host-name NOM --enable
  $( basename "${BASH_SOURCE[0]}" ) --host-name NOM --disable
  $( basename "${BASH_SOURCE[0]}" ) --profile-id ID
  $( basename "${BASH_SOURCE[0]}" ) --profile-id ID --enable
  $( basename "${BASH_SOURCE[0]}" ) --profile-id ID --disable

Actions:
  -h,       --help      Affiche ce message d’aide et quitte.
  -l,       --list      Affiche la liste des configuration au format JSON
  -A ADR,   --mac-address ADR
                        Adresse MAC d'une machine du profil réseau
  -N NOM,   --host-name NOM
                        Nom de la machine
  -P ID,    --profile-id ID
                        Identificateur du profil (nombre)

Options:
  -e,       --enable    Active l’accès internet du profil

  -d,       --disable   Désactive l’accès internet du profil

            --debug     Affiche les traces

  -k,   --insecure      Autorise les connexions non certifiées lors de l’utilisation de SSL

        --http,
        --allow-http
                        Autorise l’utilisation du protocole http (non sécurisé)

EOF
}

function __get_network_control_with_filter__ {
  local -r argname="$1"
  local -r argvalue="$2"
  local -r filter="$3"

  local network_control_json=
  network_control_json="$( "${FREEBOX_BIN}" --network-control )" || return $?

  local result=
  result="$( jq -c --arg "${argname}" "${argvalue}" <<<"${network_control_json}" "${filter}" )" || return $?
  if [ -z "${result}" ] ; then
    return 100
  fi
  echo "${result}"
}

function show_list {
  __get_network_control_with_filter__ 'X' 'X' '[ .[] |
{
  profile_id: .profile_id,
  profile_name: .profile_name,
  rule_mode: .rule_mode,
  hosts: ( .hosts | map( { default_name: .default_name, mac: .l2ident.id } ) | sort_by(.mac) )
} ] | sort_by(.profile_id)' || return $?
# hosts: ( [ .hosts[] | { default_name: .default_name, mac: .l2ident.id } ] | sort_by(.mac) )
}


function __get_network_control_profile_id_from_mac__ {
  local -r mac_addr_upper_case="$1"

  # shellcheck disable=SC2016 # Expressions don't expand in single quotes - JQ
  __get_network_control_with_filter__ 'mac' "${mac_addr_upper_case}" \
    '.[] | select( .hosts[].l2ident.id == $mac ) | .profile_id' || return $?
}

function __get_network_control_profile_id_from_host_default_name__ {
  local -r host_default_name="$1"

  # shellcheck disable=SC2016 # Expressions don't expand in single quotes - JQ
  __get_network_control_with_filter__ 'name' "${host_default_name}" \
    '.[] | select( .hosts[].default_name == $name ) | .profile_id' || return $?
}

function __network_control_apply_filter__ {
  jq -S '
  del( .cdayranges ) |
  del( .hosts[] // [] | .access_point ) |
  del( .hosts[] // [] | .host_type ) |
  del( .hosts[] // [] | .interface ) |
  del( .hosts[] // [] | .l3connectivities ) |
  del( .hosts[] // [] | .last_activity ) |
  del( .hosts[] // [] | .last_time_reachable ) |
  del( .hosts[] // [] | .names ) |
  del( .hosts[] // [] | .network_control ) |
  del( .hosts[] // [] | .persistent ) |
  del( .hosts[] // [] | .primary_name_manual ) |
  del( .hosts[] // [] | .reachable ) |
  del( .hosts[] // [] | .vendor_name ) |
  del( .next_change ) |
  del( .profile_icon ) |
  del( .resolution )
' || return $?
}

function show_internet_config_by_mac_addr {
  local -r mac_addr_upper_case="$1"

  echoinfo "${FUNCNAME[0]}( '${mac_addr_upper_case}' )"

  local -i profile_id=
  profile_id="$( __get_network_control_profile_id_from_mac__ "${mac_addr_upper_case}" )" || return $?
  local -ri profile_id

  show_internet_config_by_profile_id "${profile_id}" || return $?
}

function show_internet_config_by_host_name {
  local -r host_default_name="$1"

  echoinfo "${FUNCNAME[0]}( '${host_default_name}' )"

  local -i profile_id=
  profile_id="$( __get_network_control_profile_id_from_host_default_name__ "${host_default_name}" )" || return $?
  local -ri profile_id

  show_internet_config_by_profile_id "${profile_id}" || return $?
}

function show_internet_config_by_profile_id {
  local -ri profile_id="$1"

  echoinfo "${FUNCNAME[0]}( '${profile_id}' )"

  local result=
  result="$( "${FREEBOX_BIN}" --network-control "${profile_id}" )"|| return $?

  __network_control_apply_filter__ <<<"${result}" || return $?
}

function enable_internet_access_by_mac_addr {
  local -r enable_internet="$1"
  local -r mac_addr_upper_case="$2"

  echoinfo "${FUNCNAME[0]}( '${enable_internet}', '${mac_addr_upper_case}' )"

  local -a params=()
  if [ "${enable_internet}" = 'true' ] ; then
    params+=(--network-enable-by-mac "${mac_addr_upper_case}")
  else
    params+=(--network-disable-by-mac "${mac_addr_upper_case}")
  fi

  local result=
  result="$( "${FREEBOX_BIN}" "${params[@]}")" || return $?
  local -r result

  jq '.result' <<<"${result}" | __network_control_apply_filter__ || return $?
}

function enable_internet_access_by_host_name {
  local -r enable_internet="$1"
  local -r host_default_name="$2"

  echoinfo "${FUNCNAME[0]}( '${enable_internet}', '${host_default_name}' )"

  local -i profile_id=
  profile_id="$( __get_network_control_profile_id_from_host_default_name__ "${host_default_name}" )" || return $?
  local -ri profile_id

  enable_internet_access_by_profile_id "${enable_internet}" "${profile_id}" || return $?
}

function enable_internet_access_by_profile_id {
  local -r  enable_internet="$1"
  local -ri profile_id="$2"

  echoinfo "${FUNCNAME[0]}( '${enable_internet}', '${profile_id}' )"

  local -a params=()
  if [ "${enable_internet}" = 'true' ] ; then
    params+=(--network-enable-by-profile "${profile_id}")
  else
    params+=(--network-disable-by-profile "${profile_id}")
  fi

  local result=
  result="$( "${FREEBOX_BIN}" "${params[@]}")" || return $?
  local -r result

  jq '.result' <<<"${result}" | __network_control_apply_filter__ || return $?
}

function main {
  local param=
  local param_enable_internet_access=
  local param_mac_address=
  local param_hostname=
  local param_action_list=
  local param_profile_id=

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage 'true'
        exit 0
        ;;

      '-M'|'--mac-address')
        param_mac_address="$( P_param "$@" )" && shift || return $?
        ;;

      '-P'|'--profile-id')
        param_profile_id="$( P_param "$@" )" && shift || return $?
        ;;

      '-N'|'--host-name')
        param_hostname="$( P_param "$@" )" && shift || return $?
        ;;

      '-e'|'--enable')
        param_enable_internet_access='true'
        ;;

      '-d'|'--disable')
        param_enable_internet_access='false'
        ;;

      '-l'|'--list')
        param_action_list='true'
        ;;

      '-k'|'--insecure')
        declare -grx FREEBOX_INSECURE='true'
        ;;

      '--http'|'--allow-http')
        declare -grx FREEBOX_HTTPS_ONLY='true'
        ;;

      '--debug')
        declare -grx FREEBOX_DEBUG_CURL='true'
        ;;

      *)
        # unknown option
        echofatal "Paramètre inconnu: '${param}'."
        usage >&2
        return 1
        ;;
    esac
  done

  if [ "${param_action_list}" = 'true' ] ; then
    show_list | jq '.' || return $?
  elif [ -z "${param_mac_address}" ] && [ -z "${param_profile_id}" ] && [ -z "${param_hostname}" ]; then
    echofatal "Paramètre --mac-address manquant" ; usage >&2 ; return 1
  elif [ -n "${param_mac_address}" ] ; then
    # Use MAC ADDRESS
    if [ -n "${param_enable_internet_access}" ] ; then
      enable_internet_access_by_mac_addr "${param_enable_internet_access}" "${param_mac_address}" || return $?
    else
      show_internet_config_by_mac_addr "${param_mac_address}" || return $?
    fi
  elif [ -n "${param_hostname}" ] ; then
    # Use HOST NAME
    if [ -n "${param_enable_internet_access}" ] ; then
      enable_internet_access_by_host_name "${param_enable_internet_access}" "${param_hostname}" || return $?
    else
      show_internet_config_by_host_name "${param_hostname}" || return $?
    fi
  else
    # Use PROFILE ID
    if [ -n "${param_enable_internet_access}" ] ; then
      enable_internet_access_by_profile_id "${param_enable_internet_access}" "${param_profile_id}" || return $?
    else
      show_internet_config_by_profile_id "${param_profile_id}" || return $?
    fi
  fi
}

main "$@" || exit $?
