#!/bin/bash -e
#
# freebox-tools:bin/freebox-lcd.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

# shellcheck disable=SC1091
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/api/freebox-bash-api.sh" || exit $?

declare -r FREEBOX_BIN="${PROJECT_HOME}/bin/freebox.sh"

function usage {
  cat <<EOF
Assistant pour votre Freebox permettant de gérer le WIFI.

Usage:
  $( basename "${BASH_SOURCE[0]}" ) --help
  $( basename "${BASH_SOURCE[0]}" ) --show [--debug]
  $( basename "${BASH_SOURCE[0]}" ) --reset [--debug]
  $( basename "${BASH_SOURCE[0]}" ) --fun [--debug]

Actions:
  -h,   --help          Affiche ce message d’aide et quitte.
  -s,   --show          Affiche la configuration actuelle de l'écran LCD
  -R,   --reset         Réinitialise avec les valeurs par défaut
  -F,   --fun           On s'amuse avec l'écran LCD de la box.

Options:
  -D,   --debug         Affiche les traces

  -k,   --insecure      Autorise les connexions non certifiées lors de l’utilisation de SSL

        --http,
        --allow-http
                        Autorise l’utilisation du protocole http (non sécurisé)

EOF
}

function run_freebox {
  local result=
  result="$( "${FREEBOX_BIN}" "$@" )" || return $?

  response_check_success "${result}" || return $?
  jq '.result' <<<"${result}" || return $?
}

function get_lcd_config {
  # GET /api/v8/lcd/config/ HTTP/1.1
  run_freebox GET '/lcd/config/' || return $?
}

function _set_lcd_config_ {
  local -r json="$1"

  # PUT /api/v8/lcd/config/ HTTP/1.1
  # { "brightness": 50, "orientation": 0, "orientation_forced": false "hide_wifi_key": false }
  run_freebox PUT '/lcd/config/' --data "${json}" || return $?
}

function set_lcd_config {
  local -r brightness="$1" # integer
  local -r orientation="$2" # integer
  local -r orientation_forced="$3" # boolean
  local -r hide_wifi_key="$4" # boolean

  local data='{}'
  if [ -n "${brightness}" ] ; then
    data="$( jq -c --arg int "${brightness}" '. += { brightness: ( $int | tonumber ) }' <<<"${data}" )" || return $?
  fi
  if [ -n "${orientation}" ] ; then
    data="$( jq -c --arg int "${orientation}" '. += { orientation: ( $int | tonumber ) }' <<<"${data}" )" || return $?
  fi
  if [ -n "${orientation_forced}" ] ; then
    data="$( jq -c --arg bool "${orientation_forced}" '. += { orientation_forced: ( $bool | test("true") ) }' <<<"${data}" )" || return $?
  fi
  if [ -n "${hide_wifi_key}" ] ; then
    data="$( jq -c --arg bool "${hide_wifi_key}" '. += { hide_wifi_key: ( $bool | test("true") ) }' <<<"${data}" )" || return $?
  fi
  _set_lcd_config_ "${data}" || return $?
}

function set_default_lcd_config {
  _set_lcd_config_ '{"orientation":0,"brightness":100,"hide_wifi_key":false,"orientation_forced":false}' || return $?
}

function lcd_config_fun {
  local brightness=50 # integer
  local -i orientation=90 # integer
  local orientation_forced='true' # boolean
  local hide_wifi_key= # boolean (not set)

  trap set_default_lcd_config EXIT

  local -i i=0
  for i in {1..100} ; do
    orientation=$(( 90*(i%4) ))
    brightness=$(( (i*31)%75 + 25 ))
    set_lcd_config "${brightness}" "${orientation}" "${orientation_forced}" "${hide_wifi_key}" | jq -c -s '.' || return $?
    sleep 1
  done
}

function main {
  local param=
  local param_lcd_action=

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage 'true'
        exit 0
        ;;

      '-s'|'--show')
        param_lcd_action='show'
        ;;

      '-R'|'--reset')
        param_lcd_action='reset'
        ;;

      '-F'|'--fun')
        param_lcd_action='fun-lcd'
        ;;

      '-k'|'--insecure')
        declare -grx FREEBOX_INSECURE='true'
        ;;

      '--http'|'--allow-http')
        declare -grx FREEBOX_HTTPS_ONLY='true'
        ;;

      '-D'|'--debug')
        declare -grx FREEBOX_DEBUG_CURL='true'
        ;;

      *)
        # unknown option
        echofatal "Paramètre inconnu: '${param}'."
        usage >&2
        exit 1
        ;;
    esac
  done

  case "${param_lcd_action}" in
    'reset')
      set_default_lcd_config || return $?
      ;;

    'fun-lcd')
      lcd_config_fun || return $?
      ;;

    'show')
      get_lcd_config || return $?
      ;;

    '')
      echofatal 'Action manquante'
      usage >&2
      exit 1
      ;;
    *)
      echofatal "Action inconnue: '${param_lcd_action}'"
      usage >&2
      exit 1
      ;;
  esac
}

main "$@" || exit $?
