#!/bin/bash -e
#
# freebox-tools:bin/freebox-wifi.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

# shellcheck disable=SC1091
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit $?
# shellcheck disable=SC1091
source "${PROJECT_HOME}/bin/api/freebox-bash-api.sh" || exit $?

declare -r FREEBOX_BIN="${PROJECT_HOME}/bin/freebox.sh"

function usage {
  cat <<EOF
Assistant pour votre Freebox permettant de gérer le WIFI.

Usage:
  $( basename "${BASH_SOURCE[0]}" ) --help
  $( basename "${BASH_SOURCE[0]}" )
  $( basename "${BASH_SOURCE[0]}" ) --enable
  $( basename "${BASH_SOURCE[0]}" ) --disable

Actions:
  -h,   --help          Affiche ce message d’aide et quitte.
  -e,   --enable        Active le WIFI
  -d,   --disable       Désactive le WIFI

Options:
        --debug         Affiche les traces

  -k,   --insecure      Autorise les connexions non certifiées lors de l’utilisation de SSL

        --http,
        --allow-http
                        Autorise l’utilisation du protocole http (non sécurisé)

EOF
}

function run_freebox {
  local result=
  result="$( "${FREEBOX_BIN}" "$@" )" || return $?

  response_check_success "${result}" || return $?
  jq '.result | { enabled: .enabled }' <<<"${result}" || return $?
}

function get_wifi_config {
  # GET /api/v8/wifi/config/ HTTP/1.1
  run_freebox GET '/wifi/config/' || return $?
}

function set_wifi_config {
  local -r wifi_enabled="$1"

  local data=
  data="$( jq -c --null-input --arg enabled "${wifi_enabled}" '{ enabled: ( $enabled | test("true") ) }' )" || return $?
  # PUT /api/v8/wifi/config/ HTTP/1.1
  # { "enabled": false }
  run_freebox PUT '/wifi/config/' --data "${data}" || return $?
}

function main {
  local param=
  local param_wifi_enable=

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage 'true'
        exit 0
        ;;

      '-e'|'--enable')
        param_wifi_enable='true'
        ;;

      '-d'|'--disable')
        param_wifi_enable='false'
        ;;

      '-k'|'--insecure')
        declare -grx FREEBOX_INSECURE='true'
        ;;

      '--http'|'--allow-http')
        declare -grx FREEBOX_HTTPS_ONLY='true'
        ;;

      '--debug')
        declare -grx FREEBOX_DEBUG_CURL='true'
        ;;

      *)
        # unknown option
        echofatal "Paramètre inconnu: '${param}'."
        usage >&2
        exit 1
        ;;
    esac
  done

  if [ -n "${param_wifi_enable}" ] ; then
    set_wifi_config "${param_wifi_enable}" || return $?
  else
    get_wifi_config || return $?
  fi
}

main "$@" || exit $?
